# Copyright (c) 2002 New Information Paradigms Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.


# Subclassed MemberDataTool for storing member data
# in an exUserFolder ( XUF )


from Products.CMFCore.utils import getToolByName
from Globals import InitializeClass
from Acquisition import aq_base
from Products.CMFCore.MemberDataTool import MemberDataTool, MemberData



class XUFMemberDataTool (MemberDataTool):
    '''This tool wraps user objects, making them act as Member objects.
    '''
    meta_type = 'XUF Member Data Tool'
    title = 'XUF Member Data Tool'


    def __init__(self, id='portal_memberdata'):
        self.id = id
        MemberDataTool.__init__(self)

        
    def wrapUser(self, u):
        '''
        If possible, returns the Member object that corresponds
        to the given User object.
        '''
        id = u.getUserName()
        members = self._members
        if not members.has_key(id):
            # Get a temporary member that might be
            # registered later via registerMemberData().
            temps = self._v_temps
            if temps is not None and temps.has_key(id):
                m = temps[id]
            else:
                base = aq_base(self)
                m = XUFMemberData(base, id)  # use XUF memberdata class
                if temps is None:
                    self._v_temps = {id:m}
                else:
                    temps[id] = m
        else:
            m = members[id]
        # Return a wrapper with self as containment and
        # the user as context.
        return m.__of__(self).__of__(u)


    def loadMemberProperties(self):
        """Reloads the member data from the database in case of external
           changes to it """

        acl_users = self.acl_users
        #get all users  from XUF
        users = acl_users.getUsers()
        
        for user in users:
            un = user.getUserName()
            #make new member object out of user
            userprops = {}
            propnames = acl_users.listUserProperties(un)
            for propname in propnames:
                propval = None
                propval = acl_users.getUserProperty(propname, un)
                if propval != None:
                    userprops[propname] = propval

            member = XUFMemberData(tool=self, id=un)
            member = member.__of__(self).__of__(user)

            #call registerMemberData() to refresh the data in this tool
            self.registerMemberData(member, un)

            member.setMemberProperties(userprops)

        return "Done"
        


InitializeClass(XUFMemberDataTool)


class XUFMemberData (MemberData):
    """ Member Data wrapper for the exUserFolder based Member Data Tool"""

    #security.declarePrivate('setMemberProperties')
    def setMemberProperties(self, mapping):
        '''Sets the properties of the member.
        '''
        #just pass the properties over to acl_users.currentPropSource
        memberdatatool = self.getTool()
        propsource = memberdatatool.acl_users.currentPropSource
        username = self.getUserName()
        
        for key in mapping.keys():
            propsource.setUserProperty(key=key,
                                       value=mapping[key],
                                       username=username)
                                                             
        #let the parent class take care of self._members
        MemberData.setMemberProperties(self, mapping)


    #security.declarePrivate('setSecurityProfile')
    def setSecurityProfile(self, password=None, roles=None, domains=None):
        """Set the user's basic security profile"""

        username = self.getUserName()
        self.getTool().acl_users._doChangeUser(name=username,
                                            password=password,
                                            roles=roles)
        #XUF ignores domains so we can ignore them too

        #let the parent class take care of the non-XUF stuff        
        MemberData.setSecurityProfile(self, password=password,
                                      roles=roles, domains=domains)


    def __getitem__(self, name):
        """function for dictionary access of props through member object
        """

        if self.__dict__.has_key(name):
            return self.__dict__[name]
        object = self.getTool().acl_users.getUserProperty(key=name,
                                              username=self.getUserName())
        if object:
            return object
        raise KeyError
    
        

InitializeClass(XUFMemberData)
